import {
	wellFormedIntegerMatcher,
	wellFormedNumberMatcher
} from "@arktype/util"
import { ScopeNode, rootSchema } from "../schemaScope.js"
import { parsedDate } from "./utils/date.js"
const number = rootSchema({
	in: {
		domain: "string",
		regex: wellFormedNumberMatcher,
		description: "a well-formed numeric string"
	},
	morph: (s) => parseFloat(s)
})
const integer = rootSchema({
	in: {
		domain: "string",
		regex: wellFormedIntegerMatcher
	},
	morph: (s) => {
		// if (!isWellFormedInteger(s)) {
		// 	return errors.mustBe("a well-formed integer string")
		// }
		// const parsed = parseInt(s)
		// return Number.isSafeInteger(parsed)
		// 	? parsed
		// 	: errors.mustBe(
		// 			"an integer in the range Number.MIN_SAFE_INTEGER to Number.MAX_SAFE_INTEGER"
		// 	  )
		return parseInt(s)
	}
})
const url = rootSchema({
	in: {
		domain: "string",
		description: "a valid URL"
	},
	morph: (s) => {
		return new URL(s)
		// try {
		// 	return new URL(s)
		// } catch {
		// 	return state.mustBe("a valid URL", s, state.basePath)
		// }
	}
})
const json = rootSchema({
	in: {
		domain: "string",
		description: "a JSON-parsable string"
	},
	morph: (s) => JSON.parse(s)
})
const date = parsedDate
export const Parsing = ScopeNode.from({
	url,
	number,
	integer,
	date,
	json
})
//# sourceMappingURL=parsing.js.map
