import type { describeExpression } from "@arktype/util"
import { BaseNode, type ConstraintNode, type Node } from "../base.js"
import type { Prerequisite } from "../kinds.js"
import type { NodeCompiler } from "../shared/compile.js"
import type { BaseNodeDeclaration } from "../shared/declare.js"
import type { Disjoint } from "../shared/disjoint.js"
import type {
	ConstraintKind,
	PropKind,
	kindLeftOf
} from "../shared/implement.js"
import type { TraverseAllows, TraverseApply } from "../shared/traversal.js"
import type { Type } from "../types/type.js"
export type constraintKindLeftOf<kind extends ConstraintKind> = ConstraintKind &
	kindLeftOf<kind>
export type constraintKindOrLeftOf<kind extends ConstraintKind> =
	| kind
	| constraintKindLeftOf<kind>
type intersectConstraintKinds<
	l extends ConstraintKind,
	r extends ConstraintKind
> = Node<l | r | "unit" | "union"> | Disjoint | null
export declare const throwInvalidOperandError: (
	kind:
		| "regex"
		| "divisor"
		| "exactLength"
		| "max"
		| "min"
		| "maxLength"
		| "minLength"
		| "before"
		| "after"
		| "prop"
		| "index"
		| "sequence"
		| "predicate",
	expected: Type<any, any>,
	actual: Type<any, any>
) => never
export declare const writeInvalidOperandMessage: <
	kind extends
		| "regex"
		| "divisor"
		| "exactLength"
		| "max"
		| "min"
		| "maxLength"
		| "minLength"
		| "before"
		| "after"
		| "prop"
		| "index"
		| "sequence"
		| "predicate",
	expected extends Type<any, any>,
	actual extends Type<any, any>
>(
	kind: kind,
	expected: expected,
	actual: actual
) => `${Capitalize<kind>} operand must be ${import("@arktype/util").join<
	import("@arktype/util").unionToTuple<
		import("@arktype/util").isAny<Prerequisite<kind>> extends true
			? "any"
			: import("@arktype/util").isNever<Prerequisite<kind>> extends true
				? "never"
				: unknown extends Prerequisite<kind>
					? "unknown"
					: Prerequisite<kind> extends infer T
						? T extends Prerequisite<kind>
							? T extends object
								? import("@arktype/util").describeObject<T>
								: T extends import("@arktype/util").Stringifiable
									? import("@arktype/util").inferDomain<
											import("@arktype/util").domainOf<T>
										> extends T
										? import("@arktype/util").join<
												import("@arktype/util").unionToTuple<
													{
														boolean: "boolean"
														null: "null"
														undefined: "undefined"
														bigint: "a bigint"
														number: "a number"
														object: "an object"
														string: "a string"
														symbol: "a symbol"
													}[import("@arktype/util").domainOf<T>]
												>,
												" or ",
												""
											>
										: `${T}`
									: import("@arktype/util").join<
											import("@arktype/util").unionToTuple<
												{
													boolean: "boolean"
													null: "null"
													undefined: "undefined"
													bigint: "a bigint"
													number: "a number"
													object: "an object"
													string: "a string"
													symbol: "a symbol"
												}[import("@arktype/util").domainOf<T>]
											>,
											" or ",
											""
										>
							: never
						: never
	>,
	" | ",
	""
>} (was ${import("@arktype/util").join<
	import("@arktype/util").unionToTuple<
		import("@arktype/util").isAny<actual["infer"]> extends true
			? "any"
			: import("@arktype/util").isNever<actual["infer"]> extends true
				? "never"
				: unknown extends actual["infer"]
					? "unknown"
					: actual["infer"] extends infer T_1
						? T_1 extends actual["infer"]
							? T_1 extends object
								? import("@arktype/util").describeObject<T_1>
								: T_1 extends import("@arktype/util").Stringifiable
									? import("@arktype/util").inferDomain<
											import("@arktype/util").domainOf<T_1>
										> extends T_1
										? import("@arktype/util").join<
												import("@arktype/util").unionToTuple<
													{
														boolean: "boolean"
														null: "null"
														undefined: "undefined"
														bigint: "a bigint"
														number: "a number"
														object: "an object"
														string: "a string"
														symbol: "a symbol"
													}[import("@arktype/util").domainOf<T_1>]
												>,
												" or ",
												""
											>
										: `${T_1}`
									: import("@arktype/util").join<
											import("@arktype/util").unionToTuple<
												{
													boolean: "boolean"
													null: "null"
													undefined: "undefined"
													bigint: "a bigint"
													number: "a number"
													object: "an object"
													string: "a string"
													symbol: "a symbol"
												}[import("@arktype/util").domainOf<T_1>]
											>,
											" or ",
											""
										>
							: never
						: never
	>,
	" | ",
	""
>})`
export type writeInvalidOperandMessage<
	kind extends ConstraintKind,
	actual extends Type
> = `${Capitalize<kind>} operand must be ${describeExpression<
	Prerequisite<kind>
>} (was ${describeExpression<actual["infer"]>})`
export interface BaseConstraintDeclaration extends BaseNodeDeclaration {
	kind: ConstraintKind
}
export declare abstract class BaseConstraint<
	d extends BaseConstraintDeclaration
> extends BaseNode<d["prerequisite"], d> {
	abstract readonly impliedBasis: Type | undefined
	readonly impliedSiblings?: ConstraintNode[] | undefined
	intersect<r extends ConstraintNode>(
		r: r
	): intersectConstraintKinds<d["kind"], r["kind"]>
}
export type PrimitiveConstraintKind = Exclude<ConstraintKind, PropKind>
export declare abstract class BasePrimitiveConstraint<
	d extends BaseConstraintDeclaration
> extends BaseConstraint<d> {
	abstract traverseAllows: TraverseAllows<d["prerequisite"]>
	abstract readonly compiledCondition: string
	abstract readonly compiledNegation: string
	abstract readonly errorContext: d["errorContext"]
	traverseApply: TraverseApply<d["prerequisite"]>
	compile(js: NodeCompiler): void
}
//# sourceMappingURL=constraint.d.ts.map
