import type { NodeCompiler } from "../shared/compile.js"
import type { BaseMeta, declareNode } from "../shared/declare.js"
import type {
	TraversalContext,
	TraverseAllows,
	TraverseApply
} from "../shared/traversal.js"
import type { constrain, of } from "./ast.js"
import { BasePrimitiveConstraint } from "./constraint.js"
export interface PredicateInner<rule extends Predicate<any> = Predicate<any>>
	extends BaseMeta {
	readonly predicate: rule
}
export type NormalizedPredicateSchema = PredicateInner
export type PredicateSchema = NormalizedPredicateSchema | Predicate<any>
export type PredicateDeclaration = declareNode<{
	kind: "predicate"
	schema: PredicateSchema
	normalizedSchema: NormalizedPredicateSchema
	inner: PredicateInner
	intersectionIsOpen: true
	errorContext: {}
}>
export declare class PredicateNode extends BasePrimitiveConstraint<PredicateDeclaration> {
	static implementation: import("../shared/implement.js").nodeImplementationOf<{
		reducibleTo: "predicate"
		prerequisite: unknown
		childKind: never
		kind: "predicate"
		schema: PredicateSchema
		normalizedSchema: NormalizedPredicateSchema
		inner: PredicateInner<Predicate<any>>
		intersectionIsOpen: true
		errorContext: import("../shared/declare.js").BaseErrorContext<"predicate">
	}>
	traverseAllows: TraverseAllows
	traverseApply: TraverseApply
	compile(js: NodeCompiler): void
	readonly impliedBasis: undefined
	readonly serializedPredicate: string
	readonly compiledCondition: string
	readonly compiledNegation: string
	readonly errorContext: {
		code: "predicate"
	}
	readonly expression: string
}
export type Predicate<data = unknown> = (
	data: data,
	ctx: TraversalContext
) => boolean
export type PredicateCast<input = never, narrowed extends input = input> = (
	input: input,
	ctx: TraversalContext
) => input is narrowed
export type inferNarrow<In, predicate> = predicate extends (
	data: any,
	...args: any[]
) => data is infer narrowed
	? In extends of<unknown, infer constraints>
		? constrain<of<narrowed, constraints>, "predicate", any>
		: constrain<narrowed, "predicate", any>
	: constrain<In, "predicate", any>
//# sourceMappingURL=predicate.d.ts.map
