import { compileSerializedValue } from "@arktype/util"
import { BasePrimitiveConstraint } from "./constraint.js"
// TODO: If node contains a predicate reference that doesn't take 1 arg, we need
// to wrap it with traversal state for allows
export class PredicateNode extends BasePrimitiveConstraint {
	static implementation = this.implement({
		hasAssociatedError: true,
		collapsibleKey: "predicate",
		keys: {
			predicate: {}
		},
		normalize: (schema) =>
			typeof schema === "function" ? { predicate: schema } : schema,
		defaults: {
			description(node) {
				return `valid according to ${
					node.predicate.name || "an anonymous predicate"
				}`
			}
		},
		intersectionIsOpen: true,
		// TODO: ordering
		intersections: {
			// TODO: allow changed order to be the same type
			// as long as the narrows in l and r are individually safe to check
			// in the order they're specified, checking them in the order
			// resulting from this intersection should also be safe.
			predicate: () => null
		}
	})
	traverseAllows = this.predicate
	traverseApply = (data, ctx) => {
		if (!this.predicate(data, ctx) && !ctx.hasError())
			ctx.error(this.errorContext)
	}
	compile(js) {
		if (js.traversalKind === "Allows") {
			js.return(this.compiledCondition)
			return
		}
		js.if(`${this.compiledNegation} && !ctx.hasError()`, () =>
			js.line(`ctx.error(${this.compiledErrorContext})`)
		)
	}
	impliedBasis = undefined
	serializedPredicate = compileSerializedValue(this.predicate)
	compiledCondition = `${this.serializedPredicate}(data, ctx)`
	compiledNegation = `!${this.compiledCondition}`
	errorContext = this.createErrorContext({})
	expression = this.serializedPredicate
}
//# sourceMappingURL=predicate.js.map
