import { throwParseError } from "@arktype/util"
import { BaseConstraint } from "../constraint.js"
export class IndexNode extends BaseConstraint {
	static implementation = this.implement({
		hasAssociatedError: false,
		intersectionIsOpen: true,
		keys: {
			key: {
				child: true,
				parse: (schema, ctx) => {
					const key = ctx.$.node(schema)
					if (!key.extends(ctx.$.keywords.propertyKey))
						return throwParseError(
							writeInvalidPropertyKeyMessage(key.expression)
						)
					return key
				}
			},
			value: {
				child: true,
				parse: (schema, ctx) => ctx.$.node(schema)
			}
		},
		normalize: (schema) => schema,
		defaults: {
			description(node) {
				return `[${node.key.expression}]: ${node.value.description}`
			}
		},
		intersections: {
			index: (l, r) => l
		}
	})
	impliedBasis = this.$.keywords.object
	expression = `[${this.key.expression}]: ${this.value.expression}`
	traverseAllows = (data, ctx) =>
		Object.entries(data).every(
			(entry) =>
				!this.key.traverseAllows(entry[0], ctx) ||
				this.value.traverseAllows(entry[1], ctx)
		)
	traverseApply = (data, ctx) =>
		Object.entries(data).forEach((entry) => {
			if (this.key.traverseAllows(entry[0], ctx)) {
				this.value.traverseApply(entry[1], ctx)
			}
		})
	compile(js) {
		if (js.traversalKind === "Allows") {
			js.return(true)
		}
	}
}
export const writeInvalidPropertyKeyMessage = (indexDef) =>
	`Indexed key definition '${indexDef}' must be a string, number or symbol`
//# sourceMappingURL=index.js.map
