import { compileSerializedValue } from "@arktype/util"
import { Disjoint } from "../../shared/disjoint.js"
import { BaseConstraint } from "../constraint.js"
export class OptionalNode extends BaseConstraint {
	static implementation = this.implement({
		keys: {
			key: {},
			value: {
				child: true,
				parse: (schema, ctx) => ctx.$.parseTypeNode(schema)
			}
		},
		hasAssociatedError: false,
		intersectionIsOpen: true,
		normalize: (schema) => schema,
		defaults: {
			description(node) {
				return `${node.compiledKey}?: ${node.value.description}`
			}
		},
		intersections: {
			optional: (l, r) => {
				if (l.key !== r.key) {
					return null
				}
				const key = l.key
				const value = l.value.intersect(r.value)
				return l.$.parse("optional", {
					key,
					value:
						value instanceof Disjoint ? l.$.tsKeywords.never : value
				})
			}
		}
	})
	impliedBasis = this.$.tsKeywords.object
	serializedKey = compileSerializedValue(this.key)
	compiledKey = typeof this.key === "string" ? this.key : this.serializedKey
	expression = `${this.compiledKey}?: ${this.value}`
	traverseAllows = (data, ctx) =>
		!(this.key in data) || this.value.traverseAllows(data[this.key], ctx)
	traverseApply = (data, ctx) => {
		if (this.key in data) {
			this.value.traverseApply(data[this.key], ctx)
		}
	}
	compile(js) {
		js.if(`${this.serializedKey} in ${js.data}`, () =>
			js.checkLiteralKey(this.key, this.value)
		)
		if (js.traversalKind === "Allows") {
			js.return(true)
		}
	}
}
//# sourceMappingURL=optional.js.map
