import { compileSerializedValue } from "@arktype/util"
import { Disjoint } from "../../shared/disjoint.js"
import { BaseConstraint } from "../constraint.js"
export class PropNode extends BaseConstraint {
	static implementation = this.implement({
		hasAssociatedError: true,
		intersectionIsOpen: true,
		keys: {
			key: {},
			value: {
				child: true,
				parse: (schema, ctx) => ctx.$.node(schema)
			},
			optional: {
				// normalize { optional: false } to {}
				parse: (schema) => schema || undefined
			}
		},
		normalize: (schema) => schema,
		defaults: {
			description(node) {
				return `${node.compiledKey}${node.optional ? "?" : ""}: ${
					node.value.description
				}`
			},
			expected() {
				return "defined"
			},
			actual: () => null
		},
		intersections: {
			prop: (l, r, $) => {
				if (l.key !== r.key) {
					return null
				}
				const key = l.key
				const value = l.value.intersect(r.value)
				if (value instanceof Disjoint) {
					return value.withPrefixKey(l.compiledKey)
				}
				return $.node("prop", {
					key,
					value,
					optional: l.optional === true && r.optional === true
				})
			}
		}
	})
	required = !this.optional
	impliedBasis = this.$.keywords.object
	serializedKey = compileSerializedValue(this.key)
	compiledKey = typeof this.key === "string" ? this.key : this.serializedKey
	expression = `${this.compiledKey}${this.optional ? "?" : ""}: ${
		this.value.expression
	}`
	errorContext = Object.freeze({
		code: "prop",
		description: this.description,
		key: this.key
	})
	traverseAllows = (data, ctx) => {
		if (this.key in data) {
			// ctx will be undefined if this node doesn't have a context-dependent predicate
			ctx?.path.push(this.key)
			const allowed = this.value.traverseAllows(data[this.key], ctx)
			ctx?.path.pop()
			return allowed
		}
		return this.required
	}
	traverseApply = (data, ctx) => {
		ctx.path.push(this.key)
		if (this.key in data) {
			this.value.traverseApply(data[this.key], ctx)
		} else if (this.required) {
			ctx.error(this.errorContext)
		}
		ctx.path.pop()
	}
	compile(js) {
		const requiresContext = js.requiresContextFor(this.value)
		if (requiresContext) {
			js.line(`ctx.path.push(${this.serializedKey})`)
		}
		js.if(`${this.serializedKey} in ${js.data}`, () =>
			js.check(this.value, {
				arg: `${js.data}${js.prop(this.key)}`
			})
		)
		if (this.required) {
			js.else(() => {
				if (js.traversalKind === "Apply") {
					return js.line(`ctx.error(${this.compiledErrorContext})`)
				} else {
					if (requiresContext) {
						js.line(`ctx.path.pop()`)
					}
					return js.return(false)
				}
			})
		}
		if (requiresContext) js.line(`ctx.path.pop()`)
		else js.return(true)
	}
}
//# sourceMappingURL=prop.js.map
