import {
	DynamicBase,
	conflatenateAll,
	flatMorph,
	reference
} from "@arktype/util"
import { arrayIndexMatcherReference } from "./shared.js"
export class PropsGroup extends DynamicBase {
	inner
	$
	constructor(inner, $) {
		super(inner)
		this.inner = inner
		this.$ = $
	}
	exhaustive = this.onExtraneousKey !== undefined || this.index !== undefined
	all = conflatenateAll(this.prop, this.index, this.sequence)
	nameSet = this.prop ? flatMorph(this.prop, (i, node) => [node.key, 1]) : {}
	nameSetReference = reference(this.nameSet)
	description = describeProps(this, "description")
	expression = describeProps(this, "expression")
	literalKeys = literalPropKeysOf(this.all)
	keyofCache
	rawKeyOf() {
		if (!this.keyofCache) {
			let branches = this.$.parseUnits(...this.literalKeys).branches
			this.index?.forEach(
				({ key }) => (branches = branches.concat(key.branches))
			)
			this.keyofCache = this.$.node(branches)
		}
		return this.keyofCache
	}
	traverseAllows = (data, ctx) =>
		this.all.every((prop) => prop.traverseAllows(data, ctx))
	traverseApply = (data, ctx) =>
		this.all.forEach((prop) => prop.traverseApply(data, ctx))
	compile(js) {
		if (this.exhaustive) {
			this.compileExhaustive(js)
		} else {
			this.compileEnumerable(js)
		}
	}
	compileEnumerable(js) {
		if (js.traversalKind === "Allows") {
			this.all.forEach((node) =>
				js.if(`!${js.invoke(node)}`, () => js.return(false))
			)
		} else {
			this.all.forEach((node) => js.line(js.invoke(node)))
		}
	}
	compileExhaustive(js) {
		this.prop?.forEach((prop) => js.check(prop))
		this.sequence?.compile(js)
		if (this.sequence) js.check(this.sequence)
		js.forIn(js.data, () => {
			if (this.onExtraneousKey) {
				js.let("matched", false)
			}
			this.index?.forEach((node) => {
				js.if(
					`${js.invoke(node.key, { arg: "k", kind: "Allows" })}`,
					() => {
						if (js.traversalKind === "Allows") {
							js.if(
								`!${js.invoke(node.value, {
									arg: `${js.data}[k]`
								})}`,
								() => js.return(false)
							)
						} else {
							js.line(
								js.invoke(node.value, { arg: `${js.data}[k]` })
							)
						}
						if (this.onExtraneousKey) {
							js.set("matched", true)
						}
						return js
					}
				)
			})
			if (this.onExtraneousKey) {
				if (this.prop?.length !== 0) {
					js.line(`matched ||= k in ${this.nameSetReference}`)
				}
				if (this.sequence) {
					js.line(`matched ||= ${arrayIndexMatcherReference}.test(k)`)
				}
				// TODO: replace error
				js.if("!matched", () => js.line(`throw new Error("strict")`))
			}
			return js
		})
	}
}
const literalPropKeysOf = (all) => {
	const keys = []
	all.forEach((node) => {
		if (node.kind === "index") return
		if (node.kind === "prop") return keys.push(node.key)
		node.prevariadic.forEach((_, i) => keys.push(`${i}`))
	})
	return keys
}
const describeProps = (inner, childStringProp) => {
	if (inner.prop || inner.index) {
		const parts = inner.index?.map(String) ?? []
		inner.prop?.forEach((node) => parts.push(node[childStringProp]))
		const objectLiteralDescription = `${
			inner.onExtraneousKey ? "exact " : ""
		}{ ${parts.join(", ")} }`
		return inner.sequence
			? `${objectLiteralDescription} & ${inner.sequence.description}`
			: objectLiteralDescription
	}
	return inner.sequence?.description ?? "{}"
}
//# sourceMappingURL=props.js.map
