import { compileSerializedValue } from "@arktype/util"
import { Disjoint } from "../../shared/disjoint.js"
import { BaseConstraint } from "../constraint.js"
const intersectNamed = (l, r, $) => {
	if (l.key !== r.key) {
		return null
	}
	const key = l.key
	const value = l.value.intersect(r.value)
	if (value instanceof Disjoint) {
		return value.withPrefixKey(l.compiledKey)
	}
	return $.parse("required", {
		key,
		value
	})
}
export class RequiredNode extends BaseConstraint {
	static implementation = this.implement({
		hasAssociatedError: true,
		intersectionIsOpen: true,
		keys: {
			key: {},
			value: {
				child: true,
				parse: (schema, ctx) => ctx.$.parseTypeNode(schema)
			}
		},
		normalize: (schema) => schema,
		defaults: {
			description(node) {
				return `${node.compiledKey}: ${node.value.description}`
			},
			expected() {
				return "provided"
			},
			actual: () => null
		},
		intersections: {
			required: intersectNamed,
			optional: intersectNamed
		}
	})
	impliedBasis = this.$.tsKeywords.object
	serializedKey = compileSerializedValue(this.key)
	compiledKey = typeof this.key === "string" ? this.key : this.serializedKey
	expression = `${this.compiledKey}: ${this.value}`
	errorContext = Object.freeze({
		code: "required",
		description: this.description,
		key: this.key
	})
	traverseAllows = (data, ctx) => {
		if (this.key in data) {
			return this.value.traverseAllows(data[this.key], ctx)
		}
		return false
	}
	traverseApply = (data, ctx) => {
		if (this.key in data) {
			this.value.traverseApply(data[this.key], ctx)
		} else {
			ctx.error(this.errorContext)
		}
	}
	compile(js) {
		js.if(`${this.serializedKey} in ${js.data}`, () =>
			js.checkLiteralKey(this.key, this.value)
		).else(() =>
			js.traversalKind === "Allows"
				? js.return(false)
				: js.line(
						`${js.ctx}.error(${JSON.stringify(this.errorContext)})`
					)
		)
		if (js.traversalKind === "Allows") {
			js.return(true)
		}
	}
}
//# sourceMappingURL=required.js.map
