import { type array, type satisfy } from "@arktype/util"
import type { TypeSchema } from "../../base.js"
import type { NodeCompiler } from "../../shared/compile.js"
import type { BaseMeta, declareNode } from "../../shared/declare.js"
import type { TypeKind, nodeImplementationOf } from "../../shared/implement.js"
import type { TraverseAllows, TraverseApply } from "../../shared/traversal.js"
import type { Type } from "../../types/type.js"
import { BaseConstraint } from "../constraint.js"
export interface NormalizedSequenceSchema extends BaseMeta {
	readonly prefix?: readonly TypeSchema[]
	readonly optionals?: readonly TypeSchema[]
	readonly variadic?: TypeSchema
	readonly minVariadicLength?: number
	readonly postfix?: readonly TypeSchema[]
}
export type SequenceSchema = NormalizedSequenceSchema | TypeSchema
export interface SequenceInner extends BaseMeta {
	readonly prefix?: readonly Type[]
	readonly optionals?: readonly Type[]
	readonly variadic?: Type
	readonly minVariadicLength?: number
	readonly postfix?: readonly Type[]
}
export type SequenceDeclaration = declareNode<{
	kind: "sequence"
	schema: SequenceSchema
	normalizedSchema: NormalizedSequenceSchema
	inner: SequenceInner
	prerequisite: array
	reducibleTo: "sequence"
	childKind: TypeKind
}>
export declare class SequenceNode extends BaseConstraint<SequenceDeclaration> {
	static implementation: nodeImplementationOf<SequenceDeclaration>
	readonly impliedBasis: Type<
		unknown[],
		import("../../keywords/jsObject.js").jsObject.exports
	>
	readonly prefix: readonly Type<any, any>[]
	readonly optionals: readonly Type<any, any>[]
	readonly prevariadic: Type<any, any>[]
	readonly postfix: readonly Type<any, any>[]
	readonly isVariadicOnly: boolean
	readonly minVariadicLength: number
	readonly minLength: number
	readonly minLengthNode:
		| import("../refinements/minLength.js").MinLengthNode
		| undefined
	readonly maxLength: number | undefined
	readonly maxLengthNode:
		| import("../refinements/maxLength.js").MaxLengthNode
		| undefined
	readonly impliedSiblings:
		| (
				| import("../refinements/minLength.js").MinLengthNode
				| import("../refinements/maxLength.js").MaxLengthNode
		  )[]
		| undefined
	protected childAtIndex(data: array, index: number): Type
	traverseAllows: TraverseAllows<array>
	traverseApply: TraverseApply<array>
	compile(js: NodeCompiler): void
	readonly tuple: SequenceTuple
	readonly expression: string
}
export declare const postfixFollowingOptionalMessage =
	"A postfix required element cannot follow an optional element"
export type postfixFollowingOptionalMessage =
	typeof postfixFollowingOptionalMessage
export declare const postfixWithoutVariadicMessage =
	"A postfix element requires a variadic element"
export type postfixWithoutVariadicMessage = typeof postfixWithoutVariadicMessage
export type SequenceElementKind = satisfy<
	keyof SequenceInner,
	"prefix" | "optionals" | "variadic" | "postfix"
>
export type SequenceElement = {
	kind: SequenceElementKind
	node: Type
}
export type SequenceTuple = array<SequenceElement>
//# sourceMappingURL=sequence.d.ts.map
