import { BaseRange, parseDateLimit, parseExclusiveKey } from "./range.js"
export class AfterNode extends BaseRange {
	static implementation = this.implement({
		collapsibleKey: "rule",
		hasAssociatedError: true,
		keys: {
			rule: {
				parse: parseDateLimit,
				serialize: (schema) => schema.toISOString()
			},
			exclusive: parseExclusiveKey
		},
		normalize: (schema) =>
			typeof schema === "number" ||
			typeof schema === "string" ||
			schema instanceof Date
				? { rule: schema }
				: schema,
		defaults: {
			description(node) {
				return node.exclusive
					? `after ${node.stringLimit}`
					: `${node.stringLimit} or later`
			},
			actual: (data) => data.toLocaleString()
		},
		intersections: {
			after: (l, r) => (l.isStricterThan(r) ? l : r)
		}
	})
	impliedBasis = this.$.keywords.Date
	traverseAllows = this.exclusive
		? (data) => +data > this.numericLimit
		: (data) => +data >= this.numericLimit
}
//# sourceMappingURL=after.js.map
