import { Disjoint } from "../../shared/disjoint.js"
import { BaseRange, parseDateLimit, parseExclusiveKey } from "./range.js"
export class BeforeNode extends BaseRange {
	static implementation = this.implement({
		collapsibleKey: "rule",
		hasAssociatedError: true,
		keys: {
			rule: {
				parse: parseDateLimit,
				serialize: (schema) => schema.toISOString()
			},
			exclusive: parseExclusiveKey
		},
		normalize: (schema) =>
			typeof schema === "number" ||
			typeof schema === "string" ||
			schema instanceof Date
				? { rule: schema }
				: schema,
		defaults: {
			description(node) {
				return node.exclusive
					? `before ${node.stringLimit}`
					: `${node.stringLimit} or earlier`
			},
			actual: (data) => data.toLocaleString()
		},
		intersections: {
			before: (l, r) => (l.isStricterThan(r) ? l : r),
			after: (before, after, $) =>
				before.overlapsRange(after)
					? before.overlapIsUnit(after)
						? $.node("unit", { unit: before.rule })
						: null
					: Disjoint.from("range", before, after)
		}
	})
	impliedBasis = this.$.keywords.Date
	traverseAllows = this.exclusive
		? (data) => +data < this.numericLimit
		: (data) => +data <= this.numericLimit
}
//# sourceMappingURL=before.js.map
