import {
	BasePrimitiveConstraint,
	writeInvalidOperandMessage
} from "../constraint.js"
export class DivisorNode extends BasePrimitiveConstraint {
	static implementation = this.implement({
		collapsibleKey: "rule",
		keys: {
			rule: {}
		},
		normalize: (schema) =>
			typeof schema === "number" ? { rule: schema } : schema,
		intersections: {
			divisor: (l, r, $) =>
				$.node("divisor", {
					rule: Math.abs(
						(l.rule * r.rule) /
							greatestCommonDivisor(l.rule, r.rule)
					)
				})
		},
		hasAssociatedError: true,
		defaults: {
			description(node) {
				return node.rule === 1
					? "an integer"
					: `a multiple of ${node.rule}`
			}
		}
	})
	traverseAllows = (data) => data % this.rule === 0
	compiledCondition = `data % ${this.rule} === 0`
	compiledNegation = `data % ${this.rule} !== 0`
	impliedBasis = this.$.keywords.number
	errorContext = this.createErrorContext(this.inner)
	expression = `% ${this.rule}`
}
export const writeIndivisibleMessage = (t) =>
	writeInvalidOperandMessage("divisor", t.$.keywords.number, t)
// https://en.wikipedia.org/wiki/Euclidean_algorithm
const greatestCommonDivisor = (l, r) => {
	let previous
	let greatestCommonDivisor = l
	let current = r
	while (current !== 0) {
		previous = current
		current = greatestCommonDivisor % current
		greatestCommonDivisor = previous
	}
	return greatestCommonDivisor
}
//# sourceMappingURL=divisor.js.map
