import { Disjoint } from "../../shared/disjoint.js"
import { BasePrimitiveConstraint } from "../constraint.js"
export class ExactLengthNode extends BasePrimitiveConstraint {
	static implementation = this.implement({
		collapsibleKey: "rule",
		keys: {
			rule: {}
		},
		normalize: (schema) =>
			typeof schema === "number" ? { rule: schema } : schema,
		intersections: {
			exactLength: (l, r) =>
				new Disjoint({
					"[length]": {
						unit: {
							l: l.$.node("unit", { unit: l.rule }),
							r: r.$.node("unit", { unit: r.rule })
						}
					}
				}),
			minLength: (exactLength, minLength) =>
				(
					minLength.exclusive
						? exactLength.rule > minLength.rule
						: exactLength.rule >= minLength.rule
				)
					? exactLength
					: Disjoint.from("range", exactLength, minLength),
			maxLength: (exactLength, maxLength) =>
				(
					maxLength.exclusive
						? exactLength.rule < maxLength.rule
						: exactLength.rule <= maxLength.rule
				)
					? exactLength
					: Disjoint.from("range", exactLength, maxLength)
		},
		hasAssociatedError: true,
		defaults: {
			description(node) {
				return `exactly length ${node.rule}`
			}
		}
	})
	traverseAllows = (data) => data.length === this.rule
	compiledCondition = `data.length === ${this.rule}`
	compiledNegation = `data.length !== ${this.rule}`
	impliedBasis = this.$.keywords.lengthBoundable
	errorContext = this.createErrorContext(this.inner)
	expression = `{ length: ${this.rule} }`
}
//# sourceMappingURL=exactLength.js.map
