import { jsData } from "../../shared/compile.js"
import { Disjoint } from "../../shared/disjoint.js"
import { BasePrimitiveConstraint } from "../constraint.js"
export class LengthNode extends BasePrimitiveConstraint {
	static implementation = this.implement({
		collapseKey: "rule",
		keys: {
			rule: {}
		},
		normalize: (schema) =>
			typeof schema === "number" ? { rule: schema } : schema,
		intersections: {
			length: (l, r) =>
				new Disjoint({
					"[length]": {
						unit: {
							l: l.$.parse("unit", { unit: l.rule }),
							r: r.$.parse("unit", { unit: r.rule })
						}
					}
				}),
			minLength: (length, minLength) =>
				(
					minLength.exclusive
						? length.rule > minLength.rule
						: length.rule >= minLength.rule
				)
					? length
					: Disjoint.from("range", length, minLength),
			maxLength: (length, maxLength) =>
				(
					maxLength.exclusive
						? length.rule < maxLength.rule
						: length.rule <= maxLength.rule
				)
					? length
					: Disjoint.from("range", length, maxLength)
		},
		hasAssociatedError: true,
		defaults: {
			description(node) {
				return `exactly length ${node.rule}`
			}
		}
	})
	traverseAllows = (data) => data.length === this.rule
	compiledCondition = `${jsData}.length === ${this.rule}`
	compiledNegation = `${jsData}.length !== ${this.rule}`
	impliedBasis = this.$.lengthBoundable
	errorContext = this.createErrorContext(this.inner)
	expression = `{ length: ${this.rule}}`
}
//# sourceMappingURL=length.js.map
