import { Disjoint } from "../../shared/disjoint.js"
import { BaseRange, parseExclusiveKey } from "./range.js"
export class MaxNode extends BaseRange {
	static implementation = this.implement({
		collapsibleKey: "rule",
		hasAssociatedError: true,
		keys: {
			rule: {},
			exclusive: parseExclusiveKey
		},
		normalize: (schema) =>
			typeof schema === "number" ? { rule: schema } : schema,
		defaults: {
			description(node) {
				return `${node.exclusive ? "less than" : "at most"} ${
					node.rule
				}`
			}
		},
		intersections: {
			max: (l, r) => (l.isStricterThan(r) ? l : r),
			min: (max, min, $) =>
				max.overlapsRange(min)
					? max.overlapIsUnit(min)
						? $.node("unit", { unit: max.rule })
						: null
					: Disjoint.from("range", max, min)
		}
	})
	impliedBasis = this.$.keywords.number
	traverseAllows = this.exclusive
		? (data) => data < this.rule
		: (data) => data <= this.rule
}
//# sourceMappingURL=max.js.map
