import { Disjoint } from "../../shared/disjoint.js"
import { BaseRange, parseExclusiveKey } from "./range.js"
export class MaxLengthNode extends BaseRange {
	static implementation = this.implement({
		collapsibleKey: "rule",
		hasAssociatedError: true,
		keys: {
			rule: {},
			exclusive: parseExclusiveKey
		},
		normalize: (schema) =>
			typeof schema === "number" ? { rule: schema } : schema,
		defaults: {
			description(node) {
				return node.exclusive
					? `less than length ${node.rule}`
					: `at most length ${node.rule}`
			},
			actual: (data) => `${data.length}`
		},
		intersections: {
			maxLength: (l, r) => (l.isStricterThan(r) ? l : r),
			minLength: (max, min, $) =>
				max.overlapsRange(min)
					? max.overlapIsUnit(min)
						? $.node("exactLength", { rule: max.rule })
						: null
					: Disjoint.from("range", max, min)
		}
	})
	impliedBasis = this.$.keywords.lengthBoundable
	traverseAllows = this.exclusive
		? (data) => data.length < this.rule
		: (data) => data.length <= this.rule
}
//# sourceMappingURL=maxLength.js.map
