import { type array, type valueOf } from "@arktype/util"
import type { Node } from "../../base.js"
import type { BaseMeta, BaseNodeDeclaration } from "../../shared/declare.js"
import type { KeyDefinitions, RangeKind } from "../../shared/implement.js"
import { BasePrimitiveConstraint } from "../constraint.js"
export declare abstract class BaseRange<
	d extends BaseRangeDeclaration
> extends BasePrimitiveConstraint<d> {
	readonly boundOperandKind: {
		readonly min: "value"
		readonly max: "value"
		readonly minLength: "length"
		readonly maxLength: "length"
		readonly after: "date"
		readonly before: "date"
	}[d["kind"]]
	readonly compiledActual: string
	readonly comparator: ">" | "<" | ">=" | "<="
	readonly numericLimit: number
	readonly expression: string
	readonly compiledCondition: string
	readonly compiledNegation: string
	readonly stringLimit: string
	readonly errorContext: import("../../shared/declare.js").BaseErrorContext<
		d["kind"]
	> &
		d["inner"] extends infer T
		? {
				[k in keyof T]: (import("../../shared/declare.js").BaseErrorContext<
					d["kind"]
				> &
					d["inner"])[k]
			}
		: never
	readonly limitKind: LimitKind
	isStricterThan(r: Node<d["kind"] | pairedRangeKind<d["kind"]>>): boolean
	overlapsRange(r: Node<pairedRangeKind<d["kind"]>>): boolean
	overlapIsUnit(r: Node<pairedRangeKind<d["kind"]>>): boolean
}
export interface BaseRangeInner extends BaseMeta {
	readonly rule: number | Date
	readonly exclusive?: true
}
export type LimitSchemaValue<kind extends RangeKind = RangeKind> = kind extends
	| "before"
	| "after"
	? Date | number | string
	: number
export type LimitInnerValue<kind extends RangeKind = RangeKind> = kind extends
	| "before"
	| "after"
	? Date
	: number
export interface BaseNormalizedRangeSchema extends BaseMeta {
	readonly exclusive?: boolean
}
export type LimitKind = "lower" | "upper"
export type RelativeComparator<kind extends LimitKind = LimitKind> = {
	lower: ">" | ">="
	upper: "<" | "<="
}[kind]
export declare const negatedComparators: {
	readonly "<": ">="
	readonly "<=": ">"
	readonly ">": "<="
	readonly ">=": "<"
}
export declare const boundKindPairsByLower: {
	readonly min: "max"
	readonly minLength: "maxLength"
	readonly after: "before"
}
type BoundKindPairsByLower = typeof boundKindPairsByLower
export declare const boundKindPairsByUpper: {
	max: "min"
	maxLength: "minLength"
	before: "after"
}
type BoundKindPairsByUpper = typeof boundKindPairsByUpper
export type pairedRangeKind<kind extends RangeKind> =
	kind extends LowerBoundKind
		? BoundKindPairsByLower[kind]
		: BoundKindPairsByUpper[kind & UpperBoundKind]
export type LowerBoundKind = keyof typeof boundKindPairsByLower
export type LowerNode = Node<LowerBoundKind>
export type UpperBoundKind = valueOf<typeof boundKindPairsByLower>
export type UpperNode = Node<UpperBoundKind>
export type NumericallyBoundable = string | number | array
export type Boundable = NumericallyBoundable | Date
export declare const parseExclusiveKey: KeyDefinitions<BaseRangeDeclaration>["exclusive"]
export declare const parseDateLimit: (limit: LimitSchemaValue) => Date
export interface BaseRangeDeclaration extends BaseNodeDeclaration {
	kind: RangeKind
	inner: BaseRangeInner
	normalizedSchema: BaseNormalizedRangeSchema
}
export declare const operandKindsByBoundKind: {
	readonly min: "value"
	readonly max: "value"
	readonly minLength: "length"
	readonly maxLength: "length"
	readonly after: "date"
	readonly before: "date"
}
export declare const compileComparator: (
	kind: RangeKind,
	exclusive: boolean | undefined
) => ">" | "<" | ">=" | "<="
export type BoundOperandKind = "value" | "length" | "date"
export type LengthBoundableData = string | array
export type DateRangeKind = "before" | "after"
export declare const dateLimitToString: (limit: LimitSchemaValue) => string
export declare const writeUnboundableMessage: <root extends string>(
	root: root
) => `Bounded expression ${root} must be a number, string, Array, or Date`
export type writeUnboundableMessage<root extends string> =
	`Bounded expression ${root} must be a number, string, Array, or Date`
export {}
//# sourceMappingURL=range.d.ts.map
