import { BasePrimitiveConstraint } from "../constraint.js"
export class RegexNode extends BasePrimitiveConstraint {
	static implementation = this.implement({
		collapsibleKey: "rule",
		keys: {
			rule: {},
			flags: {}
		},
		normalize: (schema) =>
			typeof schema === "string"
				? { rule: schema }
				: schema instanceof RegExp
					? schema.flags
						? { rule: schema.source, flags: schema.flags }
						: { rule: schema.source }
					: schema,
		hasAssociatedError: true,
		intersectionIsOpen: true,
		intersections: {
			// for now, non-equal regex are naively intersected
			regex: () => null
		},
		defaults: {
			description(node) {
				return `matched by ${node.rule}`
			}
		}
	})
	instance = new RegExp(this.rule, this.flags)
	expression = `${this.instance}`
	traverseAllows = this.instance.test.bind(this.instance)
	compiledCondition = `${this.expression}.test(data)`
	compiledNegation = `!${this.compiledCondition}`
	impliedBasis = this.$.keywords.string
	errorContext = this.createErrorContext(this.inner)
}
//# sourceMappingURL=regex.js.map
