import type { MatchParser } from "../match.js"
import type { NodeParser, SchemaParser } from "../schema.js"
import type { Module, Scope, ScopeParser } from "../scope.js"
import type { inferred } from "../shared/inference.js"
import type {
	DeclarationParser,
	DefinitionParser,
	Generic,
	TypeParser
} from "../type.js"
import "./internal.js"
import { type tsPrimitive } from "./tsPrimitive.js"
import { type jsObject } from "./jsObject.js"
import { type parsing } from "./parsing.js"
import { type validation } from "./validation.js"
/** Root scopes can be inferred automatically from node definitions, but
 * explicitly typing them can improve responsiveness */
export type RootScope<exports> = Scope<{
	exports: exports
	locals: {}
	ambient: {}
}>
export type ArkResolutions = {
	exports: Ark
	locals: {}
	ambient: Ark
}
export type ParsingResolutions = {
	exports: parsing.infer
	locals: {}
	ambient: {}
}
export type TsGenericsResolutions<$ = Ark> = {
	exports: TsGenericsExports<$>
	locals: {}
	ambient: {}
}
type TsGenericsExports<$ = Ark> = {
	Record: Generic<
		["K", "V"],
		{
			"[K]": "V"
		},
		$
	>
}
export declare const tsGenerics: Module<TsGenericsResolutions<Ark>>
export declare const ark: Scope<ArkResolutions>
export declare const keywords: Module<ArkResolutions>
export interface Ark
	extends tsPrimitive.infer,
		jsObject.infer,
		validation.infer,
		TsGenericsExports {
	parse: Module<ParsingResolutions>
}
export declare const scope: ScopeParser<Ark, Ark>
export declare const type: TypeParser<Ark>
export declare const match: MatchParser<Ark>
export declare const schema: SchemaParser<Ark>
export declare const node: NodeParser<Ark>
export declare const define: DefinitionParser<Ark>
export declare const declare: DeclarationParser<Ark>
export declare namespace type {
	type cast<to> = {
		[inferred]?: to
	}
}
export {}
//# sourceMappingURL=ark.d.ts.map
