import {
	PredicateNode,
	type PredicateDeclaration
} from "./constraints/predicate.js"
import { IndexNode, type IndexDeclaration } from "./constraints/props/index.js"
import { PropNode, type PropDeclaration } from "./constraints/props/prop.js"
import {
	SequenceNode,
	type SequenceDeclaration
} from "./constraints/props/sequence.js"
import {
	DivisorNode,
	type DivisorDeclaration
} from "./constraints/refinements/divisor.js"
import { type BoundDeclarations } from "./constraints/refinements/kinds.js"
import {
	RegexNode,
	type RegexDeclaration
} from "./constraints/refinements/regex.js"
import type { NodeKind } from "./shared/implement.js"
import type { makeRootAndArrayPropertiesMutable } from "./shared/utils.js"
import { DomainNode, type DomainDeclaration } from "./types/domain.js"
import {
	IntersectionNode,
	type IntersectionDeclaration
} from "./types/intersection.js"
import {
	MorphNode,
	type MorphChildKind,
	type MorphDeclaration
} from "./types/morph.js"
import { ProtoNode, type ProtoDeclaration } from "./types/proto.js"
import { UnionNode, type UnionDeclaration } from "./types/union.js"
import { UnitNode, type UnitDeclaration } from "./types/unit.js"
export interface NodeDeclarationsByKind extends BoundDeclarations {
	domain: DomainDeclaration
	unit: UnitDeclaration
	proto: ProtoDeclaration
	union: UnionDeclaration
	morph: MorphDeclaration
	intersection: IntersectionDeclaration
	sequence: SequenceDeclaration
	divisor: DivisorDeclaration
	prop: PropDeclaration
	index: IndexDeclaration
	regex: RegexDeclaration
	predicate: PredicateDeclaration
}
export declare const nodesByKind: {
	readonly domain: typeof DomainNode
	readonly unit: typeof UnitNode
	readonly proto: typeof ProtoNode
	readonly union: typeof UnionNode
	readonly morph: typeof MorphNode
	readonly intersection: typeof IntersectionNode
	readonly divisor: typeof DivisorNode
	readonly regex: typeof RegexNode
	readonly predicate: typeof PredicateNode
	readonly prop: typeof PropNode
	readonly index: typeof IndexNode
	readonly sequence: typeof SequenceNode
	readonly min: typeof import("./constraints/refinements/min.js").MinNode
	readonly max: typeof import("./constraints/refinements/max.js").MaxNode
	readonly minLength: typeof import("./constraints/refinements/minLength.js").MinLengthNode
	readonly maxLength: typeof import("./constraints/refinements/maxLength.js").MaxLengthNode
	readonly exactLength: typeof import("./constraints/refinements/exactLength.js").ExactLengthNode
	readonly after: typeof import("./constraints/refinements/after.js").AfterNode
	readonly before: typeof import("./constraints/refinements/before.js").BeforeNode
}
export type NodesByKind = typeof nodesByKind
export type Declaration<kind extends NodeKind> = NodeDeclarationsByKind[kind]
export type Implementation<kind extends NodeKind> = NodesByKind[kind]
export type Schema<kind extends NodeKind> = Declaration<kind>["schema"]
export type NormalizedSchema<kind extends NodeKind> =
	Declaration<kind>["normalizedSchema"]
export type childKindOf<kind extends NodeKind> = Declaration<kind>["childKind"]
type ParentsByKind = {
	[k in NodeKind]: {
		[pKind in NodeKind]: k extends childKindOf<pKind> ? pKind : never
	}[NodeKind]
}
export type parentKindOf<kind extends NodeKind> = ParentsByKind[kind]
export type ioKindOf<kind extends NodeKind> = kind extends "morph"
	? MorphChildKind
	: reducibleKindOf<kind>
export type Prerequisite<kind extends NodeKind> =
	Declaration<kind>["prerequisite"]
export type reducibleKindOf<kind extends NodeKind> =
	Declaration<kind>["reducibleTo"] extends NodeKind
		? Declaration<kind>["reducibleTo"]
		: kind
export type Inner<kind extends NodeKind> = Declaration<kind>["inner"]
/** make nested arrays mutable while keeping nested nodes immutable */
export type MutableInner<kind extends NodeKind> =
	makeRootAndArrayPropertiesMutable<Inner<kind>>
export type MutableNormalizedSchema<kind extends NodeKind> =
	makeRootAndArrayPropertiesMutable<NormalizedSchema<kind>>
export type errorContext<kind extends NodeKind> = Readonly<
	Declaration<kind>["errorContext"]
>
export {}
//# sourceMappingURL=kinds.d.ts.map
