import type {
	Constructor,
	ErrorMessage,
	NonEnumerableDomain,
	array,
	conform,
	describe,
	inferDomain,
	instanceOf,
	isAny
} from "@arktype/util"
import type { Node, TypeSchema, UnknownNode } from "./base.js"
import type { Prerequisite, Schema, reducibleKindOf } from "./kinds.js"
import type { SchemaParseOptions } from "./parse.js"
import type { BasisKind, ConstraintKind, NodeKind } from "./shared/implement.js"
import type { DomainNode, DomainSchema } from "./types/domain.js"
import type {
	IntersectionNode,
	IntersectionSchema
} from "./types/intersection.js"
import type {
	Morph,
	MorphChildDefinition,
	MorphChildKind,
	MorphNode,
	MorphSchema,
	Out,
	inferMorphOut
} from "./types/morph.js"
import type { ProtoNode, ProtoSchema } from "./types/proto.js"
import type {
	NormalizedUnionSchema,
	UnionChildNode,
	UnionNode,
	UnionSchema
} from "./types/union.js"
import type { UnitNode, UnitSchema } from "./types/unit.js"
export type SchemaParser<$> = <const schema>(
	schema: validateSchema<schema, $>
) => ["schema", schema]
export type NodeParser<$> = {
	<kind extends NodeKind, const schema extends Schema<kind>>(
		kind: kind,
		schema: schema,
		opts?: SchemaParseOptions
	): Node<reducibleKindOf<kind>, instantiateSchema<schema, $>["infer"], $>
	<const schema extends TypeSchema>(
		schema: schema,
		opts?: SchemaParseOptions
	): instantiateSchema<schema, $>
}
export type validateSchema<schema, $> = schema extends UnknownNode
	? schema
	: schema extends array
		? {
				[i in keyof schema]: validateSchemaBranch<schema[i], $>
			}
		: schema extends NormalizedUnionSchema<infer branches>
			? conform<
					schema,
					NormalizedUnionSchema & {
						branches: {
							[i in keyof branches]: validateSchemaBranch<
								branches[i],
								$
							>
						}
					}
				>
			: validateSchemaBranch<schema, $>
export type instantiateSchema<schema, $> = schema extends UnionSchema<
	infer branches
>
	? branches["length"] extends 0
		? UnionNode<never, "union">
		: branches["length"] extends 1
			? instantiateSchemaBranch<branches[0], $>
			: Node<
					reducibleKindOf<"union">,
					instantiateSchemaBranch<branches[number], $>["infer"]
				>
	: instantiateSchemaBranch<schema, $>
type validateSchemaBranch<schema, $> = schema extends UnknownNode
	? schema
	: keyof schema & ("morph" | "in" | "out") extends never
		? validateMorphChild<schema, $>
		: validateMorphSchema<schema, $>
type instantiateSchemaBranch<schema, $> = schema extends UnknownNode
	? schema
	: schema extends MorphSchema
		? instantiateMorphSchema<schema, $>
		: schema extends MorphChildDefinition
			? instantiateMorphChild<schema, $>
			: UnionChildNode
type NonIntersectableBasisSchema =
	| NonEnumerableDomain
	| Constructor
	| UnitSchema
type validateMorphChild<schema, $> = [schema] extends [
	NonIntersectableBasisSchema
]
	? schema
	: validateIntersectionSchema<schema, $>
type instantiateMorphChild<schema, $> =
	schema extends NonIntersectableBasisSchema
		? instantiateBasis<schema, $>
		: schema extends IntersectionSchema
			? instantiateIntersectionSchema<schema, $>
			: Node<MorphChildKind>
type validateMorphSchema<def, $> = {
	[k in keyof def]: k extends "in" | "out"
		? validateMorphChild<def[k], $>
		: k extends keyof MorphSchema
			? MorphSchema[k]
			: `'${k & string}' is not a valid morph schema key`
}
type instantiateMorphSchema<schema, $> = schema extends MorphSchema
	? MorphNode<
			(
				In: schema["in"] extends {}
					? instantiateMorphChild<schema["in"], $>["infer"]
					: unknown
			) => schema["out"] extends {}
				? Out<instantiateMorphChild<schema["out"], $>["infer"]>
				: schema["morphs"] extends
							| Morph<any, infer o>
							| readonly [...unknown[], Morph<any, infer o>]
					? Out<inferMorphOut<o>>
					: never
		>
	: never
type exactBasisMessageOnError<schema, expected> = {
	[k in keyof schema]: k extends keyof expected
		? conform<schema[k], expected[k]>
		: ErrorMessage<
				k extends ConstraintKind
					? `${k} has a prerequisite of ${describe<Prerequisite<k>>}`
					: `'${k & string}' is not on an intersection schema`
			>
}
export type validateIntersectionSchema<schema, $> = exactBasisMessageOnError<
	schema,
	IntersectionSchema<inferBasisOf<schema, $>>
>
type inferBasisOf<schema, $> = "proto" extends keyof schema
	? instantiateBasis<conform<schema["proto"], ProtoSchema>, $>["infer"]
	: "domain" extends keyof schema
		? instantiateBasis<conform<schema["domain"], DomainSchema>, $>["infer"]
		: unknown
export type instantiateIntersectionSchema<schema, $> = keyof schema &
	ConstraintKind extends never
	? "proto" extends keyof schema
		? ProtoNode<inferBasisOf<schema, $>, $>
		: "domain" extends keyof schema
			? DomainNode<inferBasisOf<schema, $>, $>
			: IntersectionNode
	: IntersectionNode<inferBasisOf<schema, $>, $>
export type instantiateBasis<
	schema extends Schema<BasisKind>,
	$
> = isAny<schema> extends true
	? any
	: schema extends NonEnumerableDomain
		? DomainNode<inferDomain<schema>, $>
		: schema extends Constructor<infer instance>
			? ProtoNode<instance, $>
			: schema extends DomainSchema<infer domain>
				? DomainNode<inferDomain<domain>, $>
				: schema extends ProtoSchema<infer proto>
					? ProtoNode<instanceOf<proto>, $>
					: schema extends UnitSchema<infer is>
						? UnitNode<is, $>
						: never
export {}
//# sourceMappingURL=schema.d.ts.map
