import {
	type Dict,
	type List,
	type evaluate,
	type require,
	type requireKeys
} from "@arktype/util"
import type { Node, TypeNode } from "./base.js"
import type { schema } from "./builtins/builtins.js"
import type { JsObjects } from "./builtins/jsObjects.js"
import type { TsKeywords } from "./builtins/tsKeywords.js"
import type { LengthBoundableData } from "./constraints/refinements/range.js"
import type {
	instantiateAliases,
	instantiateSchemaBranches,
	validateAliases,
	validateSchemaBranch
} from "./inference.js"
import { type Schema, type reducibleKindOf } from "./kinds.js"
import { type SchemaParseOptions } from "./parse.js"
import type { TraverseAllows, TraverseApply } from "./shared/context.js"
import type {
	ActualWriter,
	ArkErrorCode,
	ExpectedWriter,
	MessageWriter,
	ProblemWriter
} from "./shared/errors.js"
import type {
	DescriptionWriter,
	NodeKind,
	TypeKind
} from "./shared/implement.js"
import type {
	NormalizedUnionSchema,
	UnionChildKind,
	UnionNode
} from "./types/union.js"
import type { UnitNode } from "./types/unit.js"
export type nodeResolutions<keywords> = {
	[k in keyof keywords]: TypeNode
}
export type BaseResolutions = Record<string, TypeNode>
declare global {
	export interface StaticArkConfig {
		preserve(): never
	}
}
type nodeConfigForKind<kind extends NodeKind> = evaluate<
	{
		description?: DescriptionWriter<kind>
	} & (kind extends ArkErrorCode
		? {
				expected?: ExpectedWriter<kind>
				actual?: ActualWriter<kind>
				problem?: ProblemWriter<kind>
				message?: MessageWriter<kind>
			}
		: {})
>
type NodeConfigsByKind = {
	[kind in NodeKind]: nodeConfigForKind<kind>
}
export type NodeConfig<kind extends NodeKind = NodeKind> =
	NodeConfigsByKind[kind]
type UnknownNodeConfig = {
	description?: DescriptionWriter
	expected?: ExpectedWriter
	actual?: ActualWriter
	problem?: ProblemWriter
	message?: MessageWriter
}
export type ParsedUnknownNodeConfig = requireKeys<
	UnknownNodeConfig,
	"description"
>
export type StaticArkOption<k extends keyof StaticArkConfig> = ReturnType<
	StaticArkConfig[k]
>
export interface ArkConfig extends Partial<NodeConfigsByKind> {}
export type ParsedArkConfig = require<ArkConfig, 2>
export declare class ScopeNode<r extends object = any> {
	def: Dict
	infer: {
		[k in keyof r]: r[k] extends schema.cast<infer t> ? t : never
	}
	static jsObjects: JsObjects.resolutions
	static tsKeywords: TsKeywords.resolutions
	readonly nodeCache: {
		[innerId: string]: Node
	}
	readonly config: ParsedArkConfig
	readonly resolutions: r
	readonly referencesByName: {
		[name: string]: Node
	}
	readonly references: readonly Node[]
	protected resolved: boolean
	protected prereducedAliases: boolean
	readonly lengthBoundable: UnionNode<LengthBoundableData>
	constructor(def: Dict, config?: ArkConfig)
	get tsKeywords(): TsKeywords.resolutions
	get jsObjects(): JsObjects.resolutions
	static from<const aliases>(
		aliases: validateAliases<aliases>,
		config?: ArkConfig
	): ScopeNode<instantiateAliases<aliases>>
	static root: ScopeNode<{}>
	parseUnion<const branches extends readonly Schema<UnionChildKind>[]>(
		input: {
			branches: {
				[i in keyof branches]: validateSchemaBranch<branches[i], r>
			}
		} & NormalizedUnionSchema
	): instantiateSchemaBranches<branches>
	parseBranches<const branches extends readonly Schema<UnionChildKind>[]>(
		...branches: {
			[i in keyof branches]: validateSchemaBranch<branches[i], r>
		}
	): instantiateSchemaBranches<branches>
	parseUnits<const branches extends List>(
		...values: branches
	): branches["length"] extends 1
		? UnionNode<branches[0]>
		: UnionNode<branches[number]> | UnitNode<branches[number]>
	parseTypeNode<defKind extends TypeKind>(
		schema: Schema<defKind>,
		opts?: TypeSchemaParseOptions<defKind>
	): Node<reducibleKindOf<defKind>>
	parseRoot<kind extends NodeKind>(
		kind: kind,
		def: Schema<kind>,
		opts?: SchemaParseOptions
	): Node<reducibleKindOf<kind>>
	parsePrereduced<kind extends NodeKind>(
		kind: kind,
		def: Schema<kind>
	): Node<kind>
	parse<kind extends NodeKind>(
		kind: kind,
		def: Schema<kind>,
		opts?: SchemaParseOptions
	): Node<reducibleKindOf<kind>>
	protected bindCompiledScope(references: readonly Node[]): void
	protected compileScope(references: readonly Node[]): {
		[k: `${string}Allows`]: TraverseAllows
		[k: `${string}Apply`]: TraverseApply
	}
	readonly schema: SchemaParser<r>
}
export type SchemaBranchesParser<$> = <
	const branches extends readonly Schema<UnionChildKind>[]
>(
	...branches: {
		[i in keyof branches]: validateSchemaBranch<branches[i], $>
	}
) => instantiateSchemaBranches<branches>
export type SchemaParser<$> = SchemaBranchesParser<$> & {
	union<const branches extends readonly Schema<UnionChildKind>[]>(
		input: {
			branches: {
				[i in keyof branches]: validateSchemaBranch<branches[i], $>
			}
		} & NormalizedUnionSchema
	): instantiateSchemaBranches<branches>
	units<const branches extends List>(
		...values: branches
	): branches["length"] extends 1
		? UnionNode<branches[0]>
		: UnionNode<branches[number]> | UnitNode<branches[number]>
}
export interface TypeSchemaParseOptions<allowedKind extends TypeKind = TypeKind>
	extends SchemaParseOptions {
	root?: boolean
	allowedKinds?: readonly allowedKind[]
}
export declare const scopeNode: typeof ScopeNode.from
export declare const rootSchema: SchemaBranchesParser<{}>
export {}
//# sourceMappingURL=schemaScope.d.ts.map
