import { type entryOf } from "@arktype/util"
import type { Node } from "../base.js"
import type {
	BoundKind,
	IntersectionChildKind,
	kindRightOf
} from "./implement.js"
type DisjointKinds = {
	domain?: {
		l: Node<"domain">
		r: Node<"domain">
	}
	unit?: {
		l: Node<"unit">
		r: Node<"unit">
	}
	proto?: {
		l: Node<"proto">
		r: Node<"proto">
	}
	presence?:
		| {
				l: true
				r: false
		  }
		| {
				l: false
				r: true
		  }
	range?: {
		l: Node<BoundKind>
		r: Node<BoundKind>
	}
	assignability?:
		| {
				l: unknown
				r: Node<IntersectionChildKind>
		  }
		| {
				l: Node<IntersectionChildKind>
				r: unknown
		  }
	union?: {
		l: readonly Node<kindRightOf<"union">>[]
		r: readonly Node<kindRightOf<"union">>[]
	}
	indiscriminableMorphs?: {
		l: Node<"union">
		r: Node<"union">
	}
	interesectedMorphs?: {
		l: Node<"morph">
		r: Node<"morph">
	}
}
export type DisjointKindEntries = entryOf<DisjointKinds>[]
export type SerializedPath = `[${string}]`
export type DisjointsSources = {
	[k in `${SerializedPath}`]: DisjointsAtPath
}
export type DisjointsAtPath = {
	[kind in DisjointKind]?: DisjointKinds[kind]
}
export type DisjointSourceEntry = entryOf<DisjointsSources>
export type FlatDisjointEntry = {
	path: SerializedPath
	kind: DisjointKind
	disjoint: Required<DisjointKinds>[DisjointKind]
}
export type DisjointKind = keyof DisjointKinds
export declare class Disjoint {
	sources: DisjointsSources
	constructor(sources: DisjointsSources)
	clone(): Disjoint
	static from<kind extends DisjointKind>(
		kind: kind,
		l: Required<DisjointKinds>[kind]["l"],
		r: Required<DisjointKinds>[kind]["r"]
	): Disjoint
	static fromEntries(entries: DisjointKindEntries): Disjoint
	get flat(): FlatDisjointEntry[]
	describeReasons(): string
	isEmpty(): boolean
	throw(): never
	invert(): Disjoint
	add(input: Disjoint): void
	withPrefixKey(key: string): Disjoint
	toString(): string
}
export {}
//# sourceMappingURL=disjoint.d.ts.map
