import {
	ReadonlyArray,
	type evaluate,
	type optionalizeKeys,
	type propwiseXor
} from "@arktype/util"
import type { Prerequisite, errorContext } from "../kinds.js"
import type { NodeKind } from "./implement.js"
import type { TraversalContext } from "./traversal.js"
import { type TraversalPath } from "./utils.js"
export declare const throwArkError: (
	message?: string | undefined,
	options?: ErrorOptions | undefined
) => never
export declare class ArkError extends TypeError {
	toString(): string
	throw(): never
}
export type ArkTypeError<code extends ArkErrorCode = ArkErrorCode> = ArkError &
	ArkErrorContext<code>
export declare const ArkTypeError: new <
	code extends ArkErrorCode = ArkErrorCode
>(
	context: ArkErrorContext<code>
) => ArkTypeError<code>
export declare class ArkErrors extends ReadonlyArray<ArkTypeError> {
	protected ctx: TraversalContext
	constructor(ctx: TraversalContext)
	byPath: Record<string, ArkTypeError>
	count: number
	private mutable
	add(error: ArkTypeError): void
	get summary(): string
	toString(): string
	throw(): never
}
export declare const createError: (
	ctx: TraversalContext,
	input: ArkErrorInput
) => ArkTypeError
export interface DerivableErrorContext<data = unknown> {
	expected: string
	actual: string | null
	problem: string
	message: string
	data: data
	path: TraversalPath
	propString: string
}
export type ArkErrorCode = {
	[kind in NodeKind]: errorContext<kind> extends null ? never : kind
}[NodeKind]
export type ArkErrorContext<code extends ArkErrorCode = ArkErrorCode> =
	errorContext<code> & DerivableErrorContext<Prerequisite<code>>
export type MessageContext<code extends ArkErrorCode = ArkErrorCode> = Omit<
	ArkErrorContext<code>,
	"message"
>
export type ProblemContext<code extends ArkErrorCode = ArkErrorCode> = Omit<
	MessageContext<code>,
	"problem"
>
type ErrorInputByCode = {
	[code in ArkErrorCode]: optionalizeKeys<
		ArkErrorContext<code>,
		keyof DerivableErrorContext
	>
}
export type CustomErrorInput = evaluate<
	{
		code?: undefined
	} & Partial<DerivableErrorContext>
>
export type ArkErrorInput =
	| string
	| ErrorInputByCode[ArkErrorCode]
	| CustomErrorInput
export type ProblemWriter<code extends ArkErrorCode = ArkErrorCode> = (
	context: ProblemContext<code>
) => string
export type MessageWriter<code extends ArkErrorCode = ArkErrorCode> = (
	context: MessageContext<code>
) => string
export type getAssociatedDataForError<code extends ArkErrorCode> =
	code extends NodeKind ? Prerequisite<code> : unknown
export type ExpectedWriter<code extends ArkErrorCode = ArkErrorCode> = (
	source: errorContext<code>
) => string
export type ActualWriter<code extends ArkErrorCode = ArkErrorCode> = (
	data: getAssociatedDataForError<code>
) => string | null
export type ArkResult<data = unknown, out = data> = propwiseXor<{
    data: data;
    out: out;
}, {
    errors: ArkErrors;
}>;
export {}
//# sourceMappingURL=errors.d.ts.map
