import { compileSerializedValue, flatMorph, printable } from "@arktype/util"
import { hasArkKind } from "../util.js"
import { throwArkError } from "./errors.js"
export const basisKinds = ["unit", "proto", "domain"]
export const propKinds = ["prop", "index", "sequence"]
export const rangeKinds = [
	"max",
	"min",
	"maxLength",
	"minLength",
	"before",
	"after"
]
export const boundKinds = ["exactLength", ...rangeKinds]
export const refinementKinds = ["regex", "divisor", ...boundKinds]
export const constraintKinds = [...refinementKinds, ...propKinds, "predicate"]
export const typeKinds = [
	"union",
	"morph",
	"unit",
	"intersection",
	"proto",
	"domain"
]
export const intersectionChildKinds = ["proto", "domain", ...constraintKinds]
export const nodeKinds = [
	...typeKinds,
	...refinementKinds,
	...propKinds,
	"predicate"
]
export const primitiveKinds = [...basisKinds, ...refinementKinds, "predicate"]
export const constraintKeys = flatMorph(constraintKinds, (i, kind) => [kind, 1])
export const propKeys = flatMorph([...propKinds, "onExtraneousKey"], (i, k) => [
	k,
	1
])
export const discriminatingIntersectionKeys = {
	...constraintKeys,
	onExtraneousKey: 1
}
export const precedenceByKind = flatMorph(nodeKinds, (i, kind) => [kind, i])
export const isNodeKind = (value) =>
	typeof value === "string" && value in precedenceByKind
export function assertNodeKind(value, kind) {
	const isNode = hasArkKind(value, "node")
	if (!isNode || value.kind !== kind)
		throwArkError(
			`Expected node of kind ${kind} (was ${
				isNode ? `${value.kind} node` : printable(value)
			})`
		)
}
export const precedenceOfKind = (kind) => precedenceByKind[kind]
export const typeKindsRightOf = (kind) =>
	typeKinds.slice(precedenceOfKind(kind) + 1)
export const defaultValueSerializer = (v) => {
	if (
		typeof v === "string" ||
		typeof v === "boolean" ||
		typeof v === "number" ||
		v === null
	) {
		return v
	}
	return compileSerializedValue(v)
}
//# sourceMappingURL=implement.js.map
