import type { ResolvedArkConfig } from "../scope.js"
import type { Morph } from "../types/morph.js"
import {
	ArkErrors,
	type ArkErrorCode,
	type ArkErrorInput,
	type ArkResult,
	type ArkTypeError
} from "./errors.js"
import type { TraversalPath } from "./utils.js"
export type QueuedMorph = {
	path: TraversalPath
	morph: Morph
}
export type BranchTraversalContext = {
	error: ArkTypeError | undefined
	morphs: QueuedMorph[]
}
export declare class TraversalContext {
	root: unknown
	config: ResolvedArkConfig
	path: TraversalPath
	morphs: QueuedMorph[]
	errors: ArkErrors
	branches: BranchTraversalContext[]
	seen: {
		[name in string]?: object[]
	}
	constructor(root: unknown, config: ResolvedArkConfig)
	get currentBranch(): BranchTraversalContext | undefined
	queueMorph(morph: Morph): void
	finalize(): ArkResult<any>
	hasError(): boolean
	get failFast(): boolean
	error<input extends ArkErrorInput>(
		input: input
	): ArkTypeError<
		input extends {
			code: ArkErrorCode
		}
			? input["code"]
			: "predicate"
	>
	get data(): unknown
	invalid(input: ArkErrorInput): false
	pushBranch(): void
	popBranch(): BranchTraversalContext
}
export type TraversalMethodsByKind<input = unknown> = {
	Allows: TraverseAllows<input>
	Apply: TraverseApply<input>
}
export type TraversalKind = keyof TraversalMethodsByKind
export type TraverseAllows<data = unknown> = (
	data: data,
	ctx: TraversalContext
) => boolean
export type TraverseApply<data = unknown> = (
	data: data,
	ctx: TraversalContext
) => void
//# sourceMappingURL=traversal.d.ts.map
