import { deepClone } from "@arktype/util"
import { ArkErrors, createError } from "./errors.js"
export class TraversalContext {
	root
	config
	path = []
	morphs = []
	errors = new ArkErrors(this)
	branches = []
	// Qualified
	seen = {}
	constructor(root, config) {
		this.root = root
		this.config = config
	}
	get currentBranch() {
		return this.branches.at(-1)
	}
	queueMorph(morph) {
		const input = { path: [...this.path], morph }
		this.currentBranch?.morphs.push(input) ?? this.morphs.push(input)
	}
	finalize() {
		if (this.hasError()) {
			return { errors: this.errors }
		}
		const data = this.root
		let out = data
		if (this.morphs.length) {
			out = deepClone(this.root)
			for (let i = 0; i < this.morphs.length; i++) {
				const { path, morph } = this.morphs[i]
				if (path.length === 0) {
					this.path = []
					// if the morph applies to the root, just assign to it directly
					const result = morph(out, this)
					if (this.hasError()) return { errors: this.errors }
					out = result
					continue
				}
				// find the object on which the key to be morphed exists
				let parent = out
				for (
					let pathIndex = 0;
					pathIndex < path.length - 1;
					pathIndex++
				)
					parent = parent[path[pathIndex]]
				// apply the morph function and assign the result to the corresponding property
				const key = path.at(-1)
				this.path = path
				const result = morph(parent[key], this)
				if (this.hasError()) return { errors: this.errors }
				else parent[key] = result
			}
		}
		return { data, out }
	}
	hasError() {
		return this.currentBranch
			? this.currentBranch.error !== undefined
			: this.errors.count !== 0
	}
	get failFast() {
		return this.branches.length !== 0
	}
	error(input) {
		const error = createError(this, input)
		if (this.currentBranch) {
			this.currentBranch.error = error
		} else {
			this.errors.add(error)
		}
		return error
	}
	get data() {
		let result = this.root
		for (const segment of this.path) {
			result = result?.[segment]
		}
		return result
	}
	invalid(input) {
		this.error(input)
		return false
	}
	pushBranch() {
		this.branches.push({
			error: undefined,
			morphs: []
		})
	}
	popBranch() {
		return this.branches.pop()
	}
}
//# sourceMappingURL=traversal.js.map
