import { domainDescriptions, domainOf, getBaseDomainKeys } from "@arktype/util"
import { Disjoint } from "../shared/disjoint.js"
import { BaseBasis } from "./basis.js"
export class DomainNode extends BaseBasis {
	static implementation = this.implement({
		hasAssociatedError: true,
		collapsibleKey: "domain",
		keys: {
			domain: {}
		},
		normalize: (input) =>
			typeof input === "string" ? { domain: input } : input,
		defaults: {
			description(node) {
				return domainDescriptions[node.domain]
			},
			actual(data) {
				// don't treat boolean as a domain since it is union of units
				// and are error messages are more consistent if they're described
				// that way universally
				return typeof data === "boolean" ? `${data}` : domainOf(data)
			}
		},
		intersections: {
			domain: (l, r) => Disjoint.from("domain", l, r)
		}
	})
	traverseAllows = (data) => domainOf(data) === this.domain
	compiledCondition =
		this.domain === "object"
			? `((typeof data === "object" && data !== null) || typeof data === "function")`
			: `typeof data === "${this.domain}"`
	compiledNegation =
		this.domain === "object"
			? `((typeof data !== "object" || data === null) && typeof data !== "function")`
			: `typeof data !== "${this.domain}"`
	errorContext = this.createErrorContext(this.inner)
	expression = this.domain
	literalKeys = getBaseDomainKeys(this.domain)
}
//# sourceMappingURL=domain.js.map
