import { type evaluate, type listable } from "@arktype/util"
import { type Node } from "../base.js"
import {
	PropsGroup,
	type ExtraneousKeyBehavior,
	type ExtraneousKeyRestriction
} from "../constraints/props/props.js"
import type { Inner, Prerequisite, Schema } from "../kinds.js"
import type { NodeCompiler } from "../shared/compile.js"
import { type BaseMeta, type declareNode } from "../shared/declare.js"
import type { ArkTypeError } from "../shared/errors.js"
import {
	type ConstraintKind,
	type IntersectionChildKind,
	type OpenNodeKind,
	type nodeImplementationOf
} from "../shared/implement.js"
import type { TraverseAllows, TraverseApply } from "../shared/traversal.js"
import type { DomainNode, DomainSchema } from "./domain.js"
import type { ProtoNode, ProtoSchema } from "./proto.js"
import { BaseType, type Type } from "./type.js"
export type IntersectionBasisKind = "domain" | "proto"
export type IntersectionInner = evaluate<
	BaseMeta & {
		domain?: DomainNode
		proto?: ProtoNode
	} & {
		[k in ConditionalIntersectionKey]?: conditionalInnerValueOfKey<k>
	}
>
export type IntersectionSchema<inferredBasis = any> = evaluate<
	BaseMeta & {
		domain?: DomainSchema
		proto?: ProtoSchema
	} & conditionalSchemaOf<inferredBasis>
>
export type IntersectionDeclaration = declareNode<{
	kind: "intersection"
	schema: IntersectionSchema
	normalizedSchema: IntersectionSchema
	inner: IntersectionInner
	reducibleTo: "intersection" | IntersectionBasisKind
	errorContext: {
		errors: readonly ArkTypeError[]
	}
	childKind: IntersectionChildKind
}>
export declare class IntersectionNode<t = unknown, $ = any> extends BaseType<
	t,
	IntersectionDeclaration,
	$
> {
	static implementation: nodeImplementationOf<IntersectionDeclaration>
	readonly basis: ProtoNode<any, any> | DomainNode<any, any> | undefined
	readonly refinements: Node<
		| "regex"
		| "divisor"
		| "exactLength"
		| "max"
		| "min"
		| "maxLength"
		| "minLength"
		| "before"
		| "after"
	>[]
	readonly props: PropsGroup | undefined
	readonly traversables: readonly (
		| PropsGroup
		| Node<
				| "proto"
				| "domain"
				| "regex"
				| "divisor"
				| "exactLength"
				| "max"
				| "min"
				| "maxLength"
				| "minLength"
				| "before"
				| "after"
				| "predicate"
		  >
	)[]
	readonly expression: string
	traverseAllows: TraverseAllows
	traverseApply: TraverseApply
	compile(js: NodeCompiler): void
	rawKeyOf(): Type
}
export type ConditionalTerminalIntersectionSchema = {
	onExtraneousKey?: ExtraneousKeyBehavior
}
export type ConditionalTerminalIntersectionInner = {
	onExtraneousKey?: ExtraneousKeyRestriction
}
type ConditionalTerminalIntersectionKey =
	keyof ConditionalTerminalIntersectionInner
type ConditionalIntersectionKey =
	| ConstraintKind
	| keyof ConditionalTerminalIntersectionInner
export type constraintKindOf<t> = {
	[k in ConstraintKind]: t extends Prerequisite<k> ? k : never
}[ConstraintKind]
type conditionalIntersectionKeyOf<t> =
	| constraintKindOf<t>
	| (t extends object ? "onExtraneousKey" : never)
type intersectionChildSchemaValueOf<k extends IntersectionChildKind> =
	k extends OpenNodeKind
		? listable<Schema<k> | Inner<k>>
		: Schema<k> | Inner<k>
type conditionalSchemaValueOfKey<k extends ConditionalIntersectionKey> =
	k extends IntersectionChildKind
		? intersectionChildSchemaValueOf<k>
		: ConditionalTerminalIntersectionSchema[k &
				ConditionalTerminalIntersectionKey]
type intersectionChildInnerValueOf<k extends IntersectionChildKind> =
	k extends OpenNodeKind ? readonly Node<k>[] : Node<k>
type conditionalInnerValueOfKey<k extends ConditionalIntersectionKey> =
	k extends IntersectionChildKind
		? intersectionChildInnerValueOf<k>
		: ConditionalTerminalIntersectionInner[k &
				ConditionalTerminalIntersectionKey]
export type conditionalSchemaOf<t> = {
	[k in conditionalIntersectionKeyOf<t>]?: conditionalSchemaValueOfKey<k>
}
export {}
//# sourceMappingURL=intersection.d.ts.map
