import {
	type BuiltinObjectKind,
	type BuiltinObjects,
	type Primitive,
	type array,
	type listable
} from "@arktype/util"
import type { Node } from "../base.js"
import type { of } from "../constraints/ast.js"
import type { Schema } from "../kinds.js"
import type { StaticArkOption } from "../scope.js"
import type { NodeCompiler } from "../shared/compile.js"
import type { BaseMeta, declareNode } from "../shared/declare.js"
import type { ArkResult, ArkTypeError } from "../shared/errors.js"
import { type nodeImplementationOf } from "../shared/implement.js"
import type {
	TraversalContext,
	TraverseAllows,
	TraverseApply
} from "../shared/traversal.js"
import { BaseType, type Type, type typeKindRightOf } from "./type.js"
export type MorphChildKind = typeKindRightOf<"morph">
export declare const morphChildKinds: readonly [
	"intersection",
	"unit",
	"proto",
	"domain"
]
export type MorphChildNode = Node<MorphChildKind>
export type MorphChildDefinition = Schema<MorphChildKind>
export type Morph<i = any, o = unknown> = (In: i, ctx: TraversalContext) => o
export type Out<o = any> = ["=>", o]
export type MorphAst<i = any, o = any> = (In: i) => Out<o>
export interface MorphInner extends BaseMeta {
	readonly in: MorphChildNode
	readonly out: MorphChildNode
	readonly morphs: readonly Morph[]
}
export interface MorphSchema extends BaseMeta {
	readonly in: MorphChildDefinition
	readonly out?: MorphChildDefinition
	readonly morphs: listable<Morph>
}
export type MorphDeclaration = declareNode<{
	kind: "morph"
	schema: MorphSchema
	normalizedSchema: MorphSchema
	inner: MorphInner
	childKind: MorphChildKind
}>
export declare class MorphNode<t = any, $ = any> extends BaseType<
	t,
	MorphDeclaration,
	$
> {
	static implementation: nodeImplementationOf<MorphDeclaration>
	readonly expression: string
	readonly serializedMorphs: `$ark.${string}`[]
	traverseAllows: TraverseAllows
	traverseApply: TraverseApply
	compile(js: NodeCompiler): void
	get in(): Node<MorphChildKind, distillConstrainableIn<t>>
	get out(): Node<MorphChildKind, distillConstrainableOut<t>>
	rawKeyOf(): Type
}
export type inferMorphOut<out> = out extends ArkResult<unknown, infer innerOut>
	? out extends null
		? out
		: innerOut
	: Exclude<out, ArkTypeError>
export type distillIn<t> = includesMorphs<t> extends true
	? distillRecurse<t, "in", "base">
	: t
export type distillOut<t> = includesMorphs<t> extends true
	? distillRecurse<t, "out", "base">
	: t
export type distillConstrainableIn<t> = includesMorphs<t> extends true
	? distillRecurse<t, "in", "constrainable">
	: t
export type distillConstrainableOut<t> = includesMorphs<t> extends true
	? distillRecurse<t, "out", "constrainable">
	: t
export type includesMorphs<t> = [
	t,
	distillRecurse<t, "in", "base">,
	t,
	distillRecurse<t, "out", "base">
] extends [
	distillRecurse<t, "in", "base">,
	t,
	distillRecurse<t, "out", "base">,
	t
]
	? false
	: true
type distillRecurse<
	t,
	io extends "in" | "out",
	distilledKind extends "base" | "constrainable"
> = unknown extends t
	? unknown
	: t extends MorphAst<infer i, infer o>
		? io extends "in"
			? i
			: o
		: t extends of<infer base, any>
			? distilledKind extends "base"
				? distillRecurse<base, io, distilledKind>
				: t
			: t extends TerminallyInferredObjectKind | Primitive
				? t
				: t extends array
					? distillArray<t, io, distilledKind, []>
					: {
							[k in keyof t]: distillRecurse<
								t[k],
								io,
								distilledKind
							>
						}
type distillArray<
	t extends array,
	io extends "in" | "out",
	constraints extends "base" | "constrainable",
	prefix extends array
> = t extends readonly [infer head, ...infer tail]
	? distillArray<
			tail,
			io,
			constraints,
			[...prefix, distillRecurse<head, io, constraints>]
		>
	: [...prefix, ...distillPostfix<t, io, constraints>]
type distillPostfix<
	t extends array,
	io extends "in" | "out",
	constraints extends "base" | "constrainable",
	postfix extends array = []
> = t extends readonly [...infer init, infer last]
	? distillPostfix<
			init,
			io,
			constraints,
			[distillRecurse<last, io, constraints>, ...postfix]
		>
	: [
			...{
				[i in keyof t]: distillRecurse<t[i], io, constraints>
			},
			...postfix
		]
/** Objects we don't want to expand during inference like Date or Promise */
type TerminallyInferredObjectKind =
	| StaticArkOption<"preserve">
	| BuiltinObjects[Exclude<BuiltinObjectKind, "Array">]
export {}
//# sourceMappingURL=morph.d.ts.map
