import {
	constructorExtends,
	getExactBuiltinConstructorName,
	objectKindDescriptions,
	objectKindOrDomainOf,
	prototypeKeysOf
} from "@arktype/util"
import { Disjoint } from "../shared/disjoint.js"
import { defaultValueSerializer } from "../shared/implement.js"
import { BaseBasis } from "./basis.js"
export class ProtoNode extends BaseBasis {
	static implementation = this.implement({
		hasAssociatedError: true,
		collapsibleKey: "proto",
		keys: {
			proto: {
				serialize: (constructor) =>
					getExactBuiltinConstructorName(constructor) ??
					defaultValueSerializer(constructor)
			}
		},
		normalize: (input) =>
			typeof input === "function" ? { proto: input } : input,
		defaults: {
			description(node) {
				const knownObjectKind = getExactBuiltinConstructorName(
					node.proto
				)
				return knownObjectKind
					? objectKindDescriptions[knownObjectKind]
					: `an instance of ${node.proto.name}`
			},
			actual(data) {
				return objectKindOrDomainOf(data)
			}
		},
		intersections: {
			proto: (l, r) =>
				constructorExtends(l.proto, r.proto)
					? l
					: constructorExtends(r.proto, l.proto)
						? r
						: Disjoint.from("proto", l, r),
			domain: (proto, domain) =>
				domain.domain === "object"
					? proto
					: // TODO: infer node to avoid cast
						Disjoint.from("domain", proto.$.keywords.object, domain)
		}
	})
	traverseAllows = (data) => data instanceof this.proto
	expression = `${this.proto.name}`
	serializedConstructor = this.json.proto
	domain = "object"
	compiledCondition = `data instanceof ${this.serializedConstructor}`
	compiledNegation = `!(${this.compiledCondition})`
	errorContext = this.createErrorContext(this.inner)
	literalKeys = prototypeKeysOf(this.proto.prototype)
}
//# sourceMappingURL=proto.js.map
