import { type Domain, type conform } from "@arktype/util"
import { BaseNode, type Node } from "../base.js"
import type { constrain } from "../constraints/ast.js"
import { type PrimitiveConstraintKind } from "../constraints/constraint.js"
import type { Predicate, inferNarrow } from "../constraints/predicate.js"
import type { Schema, reducibleKindOf } from "../kinds.js"
import type { BaseMeta, BaseNodeDeclaration } from "../shared/declare.js"
import { Disjoint } from "../shared/disjoint.js"
import {
	type ConstraintKind,
	type NodeKind,
	type TypeIntersection,
	type TypeKind,
	type kindRightOf
} from "../shared/implement.js"
import type { inferred } from "../shared/inference.js"
import type { inferIntersection } from "../shared/intersections.js"
import type { inferTypeRoot, validateTypeRoot } from "../type.js"
import type { IntersectionNode, constraintKindOf } from "./intersection.js"
import type {
	Morph,
	Out,
	distillConstrainableOut,
	includesMorphs,
	inferMorphOut
} from "./morph.js"
import type { UnionChildKind, UnionNode } from "./union.js"
export declare const defineRightwardIntersections: <
	kind extends
		| "union"
		| "morph"
		| "unit"
		| "intersection"
		| "proto"
		| "domain"
>(
	kind: kind,
	implementation: TypeIntersection<
		kind,
		Extract<
			kindRightOf<kind>,
			"union" | "morph" | "unit" | "intersection" | "proto" | "domain"
		>
	>
) => {
	[k in Extract<
		kindRightOf<kind>,
		"union" | "morph" | "unit" | "intersection" | "proto" | "domain"
	>]: TypeIntersection<kind, k>
}
export interface BaseTypeDeclaration extends BaseNodeDeclaration {
	kind: TypeKind
}
export interface Type<out t = any, $ = any>
	extends BaseType<t, BaseTypeDeclaration, $> {}
export declare abstract class BaseType<
	t,
	d extends BaseTypeDeclaration,
	$ = any
> extends BaseNode<t, d> {
	readonly branches: readonly Node<UnionChildKind>[]
	private keyofCache
	keyof(): Type<keyof this["in"]["infer"], $>
	abstract rawKeyOf(): Type
	intersect<r extends Type>(
		r: r
	): Type<inferIntersection<this["infer"], r["infer"]>, t> | Disjoint
	and<def>(
		def: validateTypeRoot<def, $>
	): Type<inferIntersection<t, inferTypeRoot<def, $>>, $>
	or<def>(def: validateTypeRoot<def, $>): Type<t | inferTypeRoot<def, $>, $>
	isUnknown(): this is IntersectionNode<unknown>
	isNever(): this is UnionNode<never>
	get<key extends PropertyKey>(...path: readonly (key | Type<key>)[]): this
	extract(other: Type): Type
	exclude(other: Type): Type
	array(): Type<t[], $>
	extends<r>(other: Type<r>): this is Type<r, $> & {
		[inferred]?: r
	}
	configure(configOrDescription: BaseMeta | string): this
	describe(description: string): this
	from(literal: this["in"]["infer"]): this["out"]["infer"]
	morph<morph extends Morph<this["infer"]>>(
		morph: morph
	): Type<
		(In: this["in"]["infer"]) => Out<inferMorphOut<ReturnType<morph>>>,
		$
	>
	morph<morph extends Morph<this["infer"]>, def>(
		morph: morph,
		outValidator: validateTypeRoot<def, $>
	): Type<
		(
			In: this["in"][typeof inferred]
		) => Out<distillConstrainableOut<inferTypeRoot<def, $>>>,
		$
	>
	narrow<def extends Predicate<distillConstrainableOut<t>>>(
		def: def
	): Type<
		includesMorphs<t> extends true
			? (In: this["in"]["infer"]) => Out<inferNarrow<this["infer"], def>>
			: inferNarrow<this["infer"], def>,
		$
	>
	assert(data: unknown): this["infer"]
	constrain<
		kind extends PrimitiveConstraintKind,
		const schema extends Schema<kind>
	>(
		kind: conform<kind, constraintKindOf<this["in"]["infer"]>>,
		schema: schema
	): Type<constrain<t, kind, schema>, $>
	protected rawConstrain(kind: ConstraintKind, schema: unknown): Type
}
export type intersectType<l extends TypeKind, r extends NodeKind> = [
	l,
	r
] extends [r, l]
	? l
	: asymmetricIntersectionOf<l, r> | asymmetricIntersectionOf<r, l>
type asymmetricIntersectionOf<
	l extends NodeKind,
	r extends NodeKind
> = l extends unknown
	? r extends kindRightOf<l>
		? l | reducibleKindOf<l>
		: never
	: never
export interface BaseBasis {
	basisName: string
	domain: Domain
}
export type typeKindRightOf<kind extends TypeKind> = Extract<
	kindRightOf<kind>,
	TypeKind
>
export type typeKindOrRightOf<kind extends TypeKind> =
	| kind
	| typeKindRightOf<kind>
export {}
//# sourceMappingURL=type.d.ts.map
