import { flatMorph, throwParseError } from "@arktype/util"
import { BaseNode } from "../base.js"
import { throwInvalidOperandError } from "../constraints/constraint.js"
import { Disjoint } from "../shared/disjoint.js"
import { typeKindsRightOf } from "../shared/implement.js"
export const defineRightwardIntersections = (kind, implementation) =>
	flatMorph(typeKindsRightOf(kind), (i, kind) => [kind, implementation])
export class BaseType extends BaseNode {
	branches = this.hasKind("union") ? this.inner.branches : [this]
	keyofCache
	keyof() {
		if (!this.keyofCache) {
			this.keyofCache = this.rawKeyOf()
			if (this.keyofCache.isNever())
				throwParseError(
					`keyof ${this.expression} results in an unsatisfiable type`
				)
		}
		return this.keyofCache
	}
	intersect(r) {
		return this.intersectInternal(r)
	}
	and(def) {
		const result = this.intersect(this.$.parseTypeRoot(def))
		return result instanceof Disjoint ? result.throw() : result
	}
	or(def) {
		const branches = [
			...this.branches,
			...this.$.parseTypeRoot(def).branches
		]
		return this.$.node(branches)
	}
	isUnknown() {
		return this.hasKind("intersection") && this.children.length === 0
	}
	isNever() {
		return this.hasKind("union") && this.branches.length === 0
	}
	get(...path) {
		return this
	}
	extract(other) {
		return this.$.node(
			this.branches.filter((branch) => branch.extends(other)),
			{ root: true }
		)
	}
	exclude(other) {
		return this.$.node(
			this.branches.filter((branch) => !branch.extends(other)),
			{ root: true }
		)
	}
	array() {
		return this.$.node(
			{
				proto: Array,
				sequence: this
			},
			{ prereduced: true, root: true }
		)
	}
	// add the extra inferred intersection so that a variable of Type
	// can be narrowed without other branches becoming never
	extends(other) {
		const intersection = this.intersect(other)
		return !(intersection instanceof Disjoint) && this.equals(intersection)
	}
	configure(configOrDescription) {
		return this.configureShallowDescendants(configOrDescription)
	}
	describe(description) {
		return this.configure(description)
	}
	// TODO: should return out
	from(literal) {
		return literal
	}
	morph(morph, outValidator) {
		if (this.hasKind("union")) {
			const branches = this.branches.map((node) =>
				node.morph(morph, outValidator)
			)
			return this.$.node("union", { ...this.inner, branches })
		}
		if (this.hasKind("morph")) {
			return this.$.node("morph", {
				...this.inner,
				morphs: [...this.morphs, morph]
			})
		}
		return this.$.node("morph", {
			in: this,
			morphs: [morph]
		})
	}
	// TODO: based on below, should maybe narrow morph output if used after
	narrow(def) {
		return this.rawConstrain("predicate", def)
	}
	assert(data) {
		const result = this(data)
		return result.errors ? result.errors.throw() : result.out
	}
	constrain(kind, schema) {
		return this.rawConstrain(kind, schema)
	}
	rawConstrain(kind, schema) {
		const constraint = this.$.node(kind, schema)
		if (constraint.impliedBasis && !this.extends(constraint.impliedBasis)) {
			return throwInvalidOperandError(kind, constraint.impliedBasis, this)
		}
		return this.and(
			// TODO: not an intersection
			this.$.node("intersection", {
				[kind]: constraint
			})
		)
	}
}
//# sourceMappingURL=type.js.map
