import type { Node } from "../base.js"
import type { Schema } from "../kinds.js"
import type { NodeCompiler } from "../shared/compile.js"
import type { BaseMeta, declareNode } from "../shared/declare.js"
import { Disjoint } from "../shared/disjoint.js"
import type { ArkTypeError } from "../shared/errors.js"
import {
	type TypeKind,
	type nodeImplementationOf
} from "../shared/implement.js"
import type { TraverseAllows, TraverseApply } from "../shared/traversal.js"
import type { Discriminant } from "./discriminate.js"
import { BaseType, type Type, type typeKindRightOf } from "./type.js"
export type UnionChildKind = typeKindRightOf<"union">
export declare const unionChildKinds: (
	| "morph"
	| "unit"
	| "intersection"
	| "proto"
	| "domain"
)[]
export type UnionChildSchema = Schema<UnionChildKind>
export type UnionChildNode = Node<UnionChildKind>
export type UnionSchema<
	branches extends readonly UnionChildSchema[] = readonly UnionChildSchema[]
> = NormalizedUnionSchema<branches> | branches
export interface NormalizedUnionSchema<
	branches extends readonly UnionChildSchema[] = readonly UnionChildSchema[]
> extends BaseMeta {
	readonly branches: branches
	readonly ordered?: true
}
export interface UnionInner extends BaseMeta {
	readonly branches: readonly UnionChildNode[]
	readonly ordered?: true
}
export type UnionDeclaration = declareNode<{
	kind: "union"
	schema: UnionSchema
	normalizedSchema: NormalizedUnionSchema
	inner: UnionInner
	errorContext: {
		errors: readonly ArkTypeError[]
	}
	reducibleTo: TypeKind
	childKind: UnionChildKind
}>
export declare class UnionNode<t = any, $ = any> extends BaseType<
	t,
	UnionDeclaration,
	$
> {
	static implementation: nodeImplementationOf<UnionDeclaration>
	readonly discriminant: Discriminant | null
	readonly isBoolean: boolean
	readonly expression: string
	traverseAllows: TraverseAllows
	traverseApply: TraverseApply
	compile(js: NodeCompiler): void
	rawKeyOf(): Type
	get nestableExpression(): string
}
export declare const intersectBranches: (
	l: readonly UnionChildNode[],
	r: readonly UnionChildNode[]
) => readonly UnionChildNode[] | Disjoint
export declare const reduceBranches: ({
	branches,
	ordered
}: UnionInner) => readonly UnionChildNode[]
//# sourceMappingURL=union.d.ts.map
