import { domainOf, printable, prototypeKeysOf } from "@arktype/util"
import { Disjoint } from "../shared/disjoint.js"
import { defaultValueSerializer } from "../shared/implement.js"
import { BaseBasis } from "./basis.js"
import { defineRightwardIntersections } from "./type.js"
export class UnitNode extends BaseBasis {
	static implementation = this.implement({
		hasAssociatedError: true,
		keys: {
			unit: {
				preserveUndefined: true,
				serialize: (v) =>
					v instanceof Date
						? v.toISOString()
						: defaultValueSerializer(v)
			}
		},
		normalize: (schema) => schema,
		defaults: {
			description(node) {
				return printable(node.unit)
			}
		},
		intersections: {
			unit: (l, r) => Disjoint.from("unit", l, r),
			...defineRightwardIntersections("unit", (l, r) =>
				r.allows(l.unit) ? l : Disjoint.from("assignability", l.unit, r)
			)
		}
	})
	traverseAllows =
		this.unit instanceof Date
			? (data) =>
					data instanceof Date &&
					data.toISOString() === this.compiledValue
			: (data) => data === this.unit
	compiledValue = this.json.unit
	serializedValue =
		typeof this.unit === "string" || this.unit instanceof Date
			? JSON.stringify(this.compiledValue)
			: this.compiledValue
	compiledCondition = compileComparison(this)
	compiledNegation = compileComparison(this, "negated")
	errorContext = this.createErrorContext(this.inner)
	expression = printable(this.unit)
	domain = domainOf(this.unit)
	literalKeys = prototypeKeysOf(this.unit)
}
const compileComparison = (unit, negated) => {
	if (unit.unit instanceof Date) {
		const condition = `data instanceof Date && data.toISOString() === ${unit.serializedValue}`
		return negated ? `!(${condition})` : condition
	}
	return `data ${negated ? "!" : "="}== ${unit.serializedValue}`
}
//# sourceMappingURL=unit.js.map
